import { ConstructorTasksIds } from "../Common/ConstructorTasksIds.js";
import { ThreadComm } from "@divinestar/threads/";
import { EreaseAndUpdate, PaintAndUpdate, VoxelUpdate, } from "./Tasks/VoxelUpdate.js";
import { WorldRegister } from "../../Data/World/WorldRegister.js";
import { TasksRequest } from "./Tasks/TasksRequest.js";
import { ChunkDataTool } from "../../Default/Tools/Data/WorldData/ChunkDataTool.js";
export default function (DVEC) {
    const chunkTool = new ChunkDataTool();
    ThreadComm.registerTasks("clear-all", () => {
        WorldRegister.instance.clearAll();
    });
    ThreadComm.registerTasks(ConstructorTasksIds.BuildChunk, async (buildData, onDone) => {
        const location = buildData.data[0];
        await DVEC.builder.buildChunk(location, buildData.data[1]);
        if (onDone)
            onDone();
    });
    ThreadComm.registerTasks(ConstructorTasksIds.BuildColumn, async (data, onDone) => {
        const column = WorldRegister.instance.column.get(data[0]);
        if (!column) {
            console.warn("Tried building a column that does not exists.", data);
            return false;
        }
        if (column.chunks.length == 0)
            return false;
        let totalChunks = 0;
        const location = data[0];
        for (const chunk of column.chunks) {
            chunkTool.setChunk(chunk);
            const chunkPOS = chunkTool.getPositionData();
            location[1] = chunkPOS.x;
            location[2] = chunkPOS.y;
            location[3] = chunkPOS.z;
            totalChunks++;
            DVEC.builder.buildChunk([...location], 1);
        }
        if (onDone)
            onDone();
    }, "deferred");
    ThreadComm.registerTasks(ConstructorTasksIds.VoxelUpdate, async (data, onDone) => {
        await VoxelUpdate(data);
        if (onDone)
            onDone();
    }, "deferred");
    ThreadComm.registerTasks(ConstructorTasksIds.VoxelErease, async (data, onDone) => {
        await EreaseAndUpdate(data);
        if (onDone)
            onDone();
    }, "deferred");
    ThreadComm.registerTasks(ConstructorTasksIds.VoxelPaint, async (data, onDone) => {
        await PaintAndUpdate(data);
        if (onDone)
            onDone();
    }, "deferred");
    ThreadComm.registerTasks(ConstructorTasksIds.Explosion, async (data) => {
        await DVEC.propagation.explosion(TasksRequest.getExplosionRequests(data[0], data[1], data[2], data[3]));
    });
    ThreadComm.registerTasks(ConstructorTasksIds.WorldSun, (data, onDone) => {
        DVEC.propagation.worldSun(TasksRequest.getWorldSunRequests(data[0], data[1]));
        if (onDone)
            onDone();
    }, "deferred");
    ThreadComm.registerTasks(ConstructorTasksIds.Generate, (data, onDone) => {
        if (!onDone)
            return;
        DVEC.worldGen.generate(data, "generate", onDone);
    }, "deferred");
    ThreadComm.registerTasks(ConstructorTasksIds.Decorate, (data, onDone) => {
        if (!onDone)
            return;
        DVEC.worldGen.generate(data, "decorate", onDone);
    }, "deferred");
    ThreadComm.registerTasks(ConstructorTasksIds.AnalyzerPropagation, async (data, onDone) => {
        await DVEC.analyzer.runPropagation(data);
        if (onDone)
            onDone();
    }, "deferred");
    ThreadComm.registerTasks(ConstructorTasksIds.AnalyzerUpdate, async (data, onDone) => {
        await DVEC.analyzer.runUpdate(data);
        if (onDone)
            onDone();
    }, "deferred");
}
