import { DivineVoxelEngineDataLoader } from "./DivineVoxelEngineDataLoader.js";
import { ThreadComm } from "@divinestar/threads/";
import { WorldRegister } from "../../Data/World/WorldRegister.js";
import { SafeInterval } from "@divinestar/utils/Intervals/SafeInterval.js";
export default function (DVEDL) {
    const DataHanlderWrapper = DVEDL.dataHandler;
    ThreadComm.registerTasks("load-region-header", async (data, onDone) => {
        const success = await DataHanlderWrapper.loadRegionHeader(data);
        return onDone ? onDone(success) : false;
    }, "deferred");
    ThreadComm.registerTasks("save-column", async (data, onDone) => {
        await DataHanlderWrapper.saveColumn(data);
        return onDone ? onDone() : false;
    }, "deferred");
    ThreadComm.registerTasks("load-column", async (data, onDone) => {
        if (WorldRegister.instance.column.get(data)) {
            if (onDone) {
                onDone();
            }
            return;
        }
        await DataHanlderWrapper.loadColumn(data);
        const inte = new SafeInterval().setInterval(1).setOnRun(() => {
            if (WorldRegister.instance.column.get(data)) {
                onDone ? onDone(true) : false;
                inte.stop();
            }
        });
        inte.start();
    }, "deferred");
    ThreadComm.registerTasks("unload-column", async (data, onDone) => {
        if (!WorldRegister.instance.column.get(data)) {
            if (onDone)
                onDone();
            return;
        }
        await DataHanlderWrapper.unLoadColumn(data);
        DivineVoxelEngineDataLoader.instance.threads.world.runPromiseTasks("unload-column", data, [], () => {
            if (onDone)
                onDone();
        });
    }, "deferred");
    ThreadComm.registerTasks("set-path", async (data, onDone) => {
        await DataHanlderWrapper.setPath(data[0]);
        return onDone ? onDone() : false;
    }, "deferred");
    ThreadComm.registerTasks("column-exists", async (data, onDone) => {
        if (WorldRegister.instance.column.get(data)) {
            if (onDone) {
                onDone();
            }
            return;
        }
        const exists = await DataHanlderWrapper.columnExists(data);
        if (onDone) {
            onDone(exists);
        }
        return false;
    }, "deferred");
    ThreadComm.registerTasks("column-timestamp", async (data, onDone) => {
        const time = await DataHanlderWrapper.columnTimestamp(data);
        if (onDone) {
            onDone(time);
        }
        return 0;
    }, "deferred");
}
