import { LocationData } from "@divinevoxel/core/Math/index.js";
import { ChunkProcessor } from "./Processors/ChunkProcessor.js";
import { DVEBuilder } from "../../Interfaces/Builder/DVEBuilder.js";
import { BuildNodeMesh } from "./Tasks/BuidlerTasks.types.js";
import { VoxelConstructor } from "./Constructors/Voxel/Classes/VoxelConstructor.js";
export type DVEDefaultBuilderInitData = {
    constructors: VoxelConstructor[];
};
export declare class DVEDefaultBuilder extends DVEBuilder {
    static instance: DVEDefaultBuilder;
    static get defaults(): {
        box: {
            simple(id: string, textures: import("Textures/Constructor.types.js").ConstructorTextureData | Record<import("@divinevoxel/core").DirectionNames, import("Textures/Constructor.types.js").ConstructorTextureData>): import("Default/Builder/Constructors/Voxel/Classes/Box/SimpleBox.constructor.js").SimpleBoxVoxelConstructor;
            pillar(id: string, textures: import("Default/Builder/Constructors/Voxel/Classes/Box/PillarBox.constructor.js").PillarBoxVoxelConstructorData): import("Default/Builder/Constructors/Voxel/Classes/Box/PillarBox.constructor.js").PillarBoxVoxelConstructor;
        };
        stair: {
            simple(id: string, texture: import("Textures/Constructor.types.js").ConstructorTextureData): import("Default/Builder/Constructors/Voxel/Classes/Stair/SimpleStair.constructor.js").SimpleStairVoxelConstructor;
        };
        panel: {
            simple(id: string, texture: import("Textures/Constructor.types.js").ConstructorTextureData): import("Default/Builder/Constructors/Voxel/Classes/Panel/SimplePanel.constructor.js").SimplePanelVoxelConstructor;
        };
        crossedPanel: {
            simple(id: string, texture: import("Textures/Constructor.types.js").ConstructorTextureData): import("Default/Builder/Constructors/Voxel/Classes/Panel/SimpleCrossedPanel.constructor.js").SimpleCrossedPanelVoxelConstructor;
        };
        liquid: {
            simple(id: string, textures: [import("Textures/Constructor.types.js").ConstructorTextureData, import("Textures/Constructor.types.js").ConstructorTextureData]): import("Default/Builder/Constructors/Voxel/Classes/Liquid/SimpleLiquid.constructor.js").SimpleLiquidConstructor;
        };
    };
    constructors: {
        constructors: import("Util/UtilMap.js").UtilMap<string, VoxelConstructor>;
        get(id: string): VoxelConstructor;
        registerVoxel(voxel: VoxelConstructor | VoxelConstructor[]): void;
        defaults: {
            box: {
                simple(id: string, textures: import("Textures/Constructor.types.js").ConstructorTextureData | Record<import("@divinevoxel/core").DirectionNames, import("Textures/Constructor.types.js").ConstructorTextureData>): import("Default/Builder/Constructors/Voxel/Classes/Box/SimpleBox.constructor.js").SimpleBoxVoxelConstructor;
                pillar(id: string, textures: import("Default/Builder/Constructors/Voxel/Classes/Box/PillarBox.constructor.js").PillarBoxVoxelConstructorData): import("Default/Builder/Constructors/Voxel/Classes/Box/PillarBox.constructor.js").PillarBoxVoxelConstructor;
            };
            stair: {
                simple(id: string, texture: import("Textures/Constructor.types.js").ConstructorTextureData): import("Default/Builder/Constructors/Voxel/Classes/Stair/SimpleStair.constructor.js").SimpleStairVoxelConstructor;
            };
            panel: {
                simple(id: string, texture: import("Textures/Constructor.types.js").ConstructorTextureData): import("Default/Builder/Constructors/Voxel/Classes/Panel/SimplePanel.constructor.js").SimplePanelVoxelConstructor;
            };
            crossedPanel: {
                simple(id: string, texture: import("Textures/Constructor.types.js").ConstructorTextureData): import("Default/Builder/Constructors/Voxel/Classes/Panel/SimpleCrossedPanel.constructor.js").SimpleCrossedPanelVoxelConstructor;
            };
            liquid: {
                simple(id: string, textures: [import("Textures/Constructor.types.js").ConstructorTextureData, import("Textures/Constructor.types.js").ConstructorTextureData]): import("Default/Builder/Constructors/Voxel/Classes/Liquid/SimpleLiquid.constructor.js").SimpleLiquidConstructor;
            };
        };
    };
    textureManager: {
        textureDataHasBeenSet: boolean;
        data: import("Textures/Texture.types.js").TextureTypeUVMap;
        getTextureUV(data: import("Textures/Constructor.types.js").ConstructorTextureData, overlay?: boolean): number;
        setTextureIndex(data: import("Textures/Texture.types.js").TextureTypeUVMap): void;
        releaseTextureData(): void;
        isReady(): boolean;
    };
    chunkProcessor: ChunkProcessor;
    nodes: {
        builders: Map<string, import("Default/Builder/Nodes/Classes/NodeBuilder.js").NodeBuilder>;
        registerBuilder(builder: import("Default/Builder/Nodes/Classes/NodeBuilder.js").NodeBuilder): void;
        buildNode(data: BuildNodeMesh): false | [import("./Tasks/BuidlerTasks.types.js").SetNodeMesh, ArrayBuffer[]];
    };
    overrides: {
        overrides: Record<import("Default/Builder/Types/Override.types.js").OverrideTypes, Map<string, Map<string, (data: import("Default/Builder/Types/Override.types.js").FaceDataOverride) => boolean>>>;
        registerOverride(type: import("Default/Builder/Types/Override.types.js").OverrideTypes, subjectId: string, neighborShapeId: string, run: (data: import("Default/Builder/Types/Override.types.js").FaceDataOverride) => boolean): void;
        hasOverride(type: import("Default/Builder/Types/Override.types.js").OverrideTypes, shapeId: string, neighborShapeId: string): boolean;
        runOverride(type: import("Default/Builder/Types/Override.types.js").OverrideTypes, firstId: string, secondOverride: string, data: import("Default/Builder/Types/Override.types.js").FaceDataOverride): boolean;
    };
    renderedSubstances: {
        meshers: import("Util/UtilMap.js").UtilMap<string, import("Default/Builder/Tools/VoxelMesherDataTool.js").VoxelMesherDataTool>;
        add(id: string): void;
    };
    constructor(data: DVEDefaultBuilderInitData);
    init(): void;
    buildChunk(location: LocationData, LOD?: number): boolean;
}
