import { Generator } from "./Classes/Generator.js";
import { RegisterDefaultTasks } from "./Tasks/RegisterDefaultTasks.js";
import { WorldSpaces } from "@divinevoxel/core/Data/World/WorldSpaces.js";
import { IWGInitalLoad } from "./Load/InitLoad.js";
import { IWGSafeExit } from "./Load/SafeExit.js";
import { IWGTeleport } from "./Load/Teleport.js";
export const IWG = {
    tasks: {
        saving: new Map(),
        updating: new Map(),
        "world-gen": new Map(),
    },
    inProgressMap: {
        map: new Map(),
        add(x, y, z, tasks) {
            this.map.set(IWG.getKey(x, y, z), tasks);
        },
        has(x, y, z) {
            return this.map.has(IWG.getKey(x, y, z));
        },
        remove(x, y, z) {
            return this.map.delete(IWG.getKey(x, y, z));
        },
    },
    getKey(x, y, z) {
        return WorldSpaces.column.getKeyXYZ(x, y, z);
    },
    generators: new Map(),
    addGenerator(data) {
        const gen = new Generator(data);
        this.generators.set(gen.data.id, gen);
        return gen;
    },
    getGenerator(id) {
        const gen = this.generators.get(id);
        if (!gen)
            return false;
        return gen;
    },
    removeGenerator(id) {
        return this.generators.delete(id);
    },
    worldGenUpdate(max = 5) {
        for (const [key, gen] of this.generators) {
            gen.worldGenUpdate(max);
        }
    },
    searchUpdate() {
        for (const [key, gen] of this.generators) {
            gen.searchUpdate();
        }
    },
    saveUpdate(max = 5) {
        for (const [key, gen] of this.generators) {
            gen.saveUpdate(max);
        }
    },
    registerTasks(type, tasksData) {
        this.tasks[type].set(tasksData.id, tasksData);
    },
    async initalLoad(dimension, position, settings = {
        renderDistance: 100,
        generateDistance: 150,
    }, onCheck = (gen) => { }) {
        const tempGen = this.addGenerator({
            id: "init-gen",
            positionWatch: position,
            ...settings,
        });
        tempGen.updateDimension(dimension);
        const loader = new IWGInitalLoad(tempGen);
        await loader.run(onCheck);
        this.removeGenerator(tempGen.data.id);
    },
    async safeExit(genId, onCheck = (gen) => { }) {
        const gen = this.getGenerator(genId);
        if (!gen)
            return false;
        const loader = new IWGSafeExit(gen);
        await loader.run(onCheck);
        this.removeGenerator(gen.data.id);
    },
    async teleport(genId, location, onCheck = (gen) => { }) {
        const gen = this.getGenerator(genId);
        if (!gen)
            return false;
        const loader = new IWGTeleport(gen, location);
        await loader.run(onCheck);
    },
};
RegisterDefaultTasks();
