import type { PhysicsNode } from "../Nodes/PhysicsNodes";
import { Vector3 } from "@divinevoxel/core/Math/Classes/Vector3.js";
import { BoundingBox } from "../Classes/BoundingBox.js";
import { CollisionNode } from "../Classes/CollisionNode.js";
import { CollisionResult } from "../Classes/CollisionResult.js";
import { Line } from "../Classes/Line.js";
import { Plane } from "../Classes/Plane.js";
/** # CollisionsHanlder
 * Handles collision handling for physics nodes.
 * ***
 *
 * The swept AABB code was adapted from this article:
 * https://luisreis.net/blog/aabb_collision_handling/
 */
export declare const CollisionsHanlder: {
    readonly COLLISION_CHECK_POSITION_OFFSET: number;
    aabb: {
        start: Vector3;
        delta: Vector3;
        results: CollisionResult;
        line: Line;
        plane: Plane;
        dimensions: Vector3;
    };
    /** # sweepAABBN
     * Calculates the collision for physics node against a bounding box.
     * @param physicsNodePosition
     * @param boundingBox
     * @param collisionNode
     * @param velocity Delata aka velocity of the physics object
     * @returns
     */
    sweepAABBN(physicsNodePosition: Vector3, boundingBox: BoundingBox, collisionNode: CollisionNode, velocity: Vector3): {
        hitDepth: number;
        nx: number;
        ny: number;
        nz: number;
    };
    between(x: number, a: number, b: number): boolean;
    processSwpetAABB(node: PhysicsNode): void;
};
