import type { TextureData } from "./Texture.types";
import { URIShader } from "@divinestar/uri/Shaders/Classes/URIShader";
import { URIMaterial } from "@divinestar/uri/Materials/URIMaterial";
import { URITexture } from "@divinestar/uri/Textures/URITexture";
declare class TextureSegment {
    parentID: string;
    id: string;
    mode: "sampler" | "overlay";
    attributeID: string;
    totalTextures: number;
    textureMap: Map<string, TextureData>;
    textures: TextureData[];
    textureIndex: Record<string, number>;
    animationsMap: number[][];
    animationTimes: number[][];
    animations: {
        uniformIndex: number;
        overlay?: boolean;
        keys: number[];
        currentFrame: number;
        currentCount: number;
        keyCounts: number[];
    }[];
    varyingID: string;
    animationUniforID: string;
    animationUniform: Float32Array;
    paths: Map<string, false | Uint8ClampedArray>;
    shaderTexture: URITexture<any, any> | null;
    textureID: string;
    constructor(parentID: string, id: string, mode: "sampler" | "overlay", attributeID: string);
    /**# clearData
     * ---
     * Clear all un-needed data.
     */
    clearData(): void;
    /**# flush
     * ---
     * Clear all data.
     */
    flush(): void;
}
export declare class TextureType {
    id: string;
    extension: string;
    segments: Map<string, TextureSegment>;
    materials: Map<string, URIMaterial>;
    shader: URIShader;
    constructor(id: string);
    flushAll(): void;
    build(): Promise<void>;
    getTextureIndex(textureId: string, varation?: string, segment?: string): number;
    clearSegmentData(): void;
    removeSegment(id: string): false | undefined;
    addTexture(data: TextureData): false | undefined;
    addToShader(shader: URIShader): URIShader;
    addToMaterial(material: URIMaterial): void;
    runAnimations(): void;
    getTextureIndexMap(): Record<string, Record<string, number>>;
    _processVariations(textureData: TextureData, paths: Map<string, Uint8ClampedArray | false>, map: Record<string, number>, animations: number[][], textureAnimatioTimes: number[][], extension: string, count: number): number;
    _getPath(textureData: TextureData, varation: string | undefined, extension: string): string;
    buildTextureIndex(): false | undefined;
}
export {};
